import datetime
import json
import os
import requests


from .constants import DISCOVER_CREDS_FOLDER_PATH
from .env import CONSTANTS_URL, TOKENS_URL, S3_DATALAKE_BUCKET
from .helpers import make_discover_request, create_folder


def boto3_client(repo_id, service='s3'):
    import boto3
    tokens = get_tokens(repo_id)
    service = boto3.client(
            service,
            aws_access_key_id = tokens['AccessKeyId'],
            aws_secret_access_key = tokens['SecretAccessKey'],
            aws_session_token = tokens['SessionToken']
    )
    return service
    
def download_file(key, repo_id, local_fp):
    bucket = S3_DATALAKE_BUCKET
    s3 = boto3_client(repo_id, 's3')
    s3.download_file(bucket, key, local_fp)

def request_tokens(repo_id):
    print('------requesting new tokens------')
    resp = make_discover_request('POST', TOKENS_URL.format(repo_id))
    return resp.json()['data']['attributes']

def get_tokens(repo_id):
    TOKENS_CREDS_FILEPATH = DISCOVER_CREDS_FOLDER_PATH+f'/{repo_id}'
    try:
        print('--------looking for existing tokens-------')
        fp = open(TOKENS_CREDS_FILEPATH, 'rb')
        creds = json.load(fp)
        exp = creds['tokens']['Expiration'].split('+')[0]
        exp = datetime.datetime.strptime(exp, '%Y-%m-%dT%H:%M:%S')
        if exp<=datetime.datetime.now():
            raise Exception('Token expired')
        return creds['tokens']
    except Exception as e:
        print('------error while fetching existing tokens-----')
        tokens = request_tokens(repo_id)
        creds = {'tokens' : tokens}
        create_folder(DISCOVER_CREDS_FOLDER_PATH)
        fp = open(TOKENS_CREDS_FILEPATH, 'w')
        json.dump(creds, fp)
        fp.close()
        return tokens
        
        
            
    